@@ -234,11 +234,13 @@ public:
 	bool done;
 
 	static double REGEN_RATE;
+	static int ACTION_SPACE;
 	static vector<AssetStats> assetList;
 	static Names n;
 
 	ProtossMacroEnv() {
 		REGEN_RATE = 1;
+		ACTION_SPACE = 68;
 		loadStatCatalog();
 		for (int i = 0; i < assetList.size(); i++) {
 			state.push_back(0);
@@ -432,6 +434,23 @@ public:
 		//done = isDone();
 	}
 
+	void step(int action) {
+		//assume default executor and chrono target
+		vector<int> viableExecutors = getViableExecutors(action);
+		if (action == n.chronoboost) {
+			int executor = pickNextExecutor(viableExecutors);
+			vector<int> viableTargets = getViableTargets(action, executor);
+			int target = viableTargets[rand() % viableTargets.size()];
+			step(action, executor, target);
+
+		}
+		else {
+			int executor = pickNextExecutor(viableExecutors);
+			step(action, executor, -1);
+		}
+
+	}
+
 	double getTimeNextSupplyDone() {
 		double minTime = 10000;
 		for (const Entity& e : entityList) {
@@ -562,7 +581,6 @@ public:
 				nextComplete = i;
 			}
 		}
-
 		return nextComplete;
 	}
 
@@ -576,6 +594,16 @@ public:
 		return true;
 	}
 
+	double getTimeAllComplete() {
+		double timeAllComplete = time;
+		for (const Entity& e : entityList) {
+			if (e.isComplete == false) {
+				timeAllComplete = max(e.timeToComplete, timeAllComplete);
+			}
+		}
+		return timeAllComplete;
+	}
+
 	bool techExists(int i) {
 		for (int req : assetList[i].techReqs) {
 			if (state[req] == 0) {
@@ -620,6 +648,21 @@ public:
 		viableActions.resize(viableCount);
 		return viableActions;
 	}
+
+	vector<int> getEssentialActions() {
+		//return all viable actions where goal state > current state
+		vector<int> viableActions = getViableActions();
+		vector<int > essentialActions(100);
+		int essentialCount = 0;
+		for (int i : viableActions) {
+			if (i != n.chronoboost && goalState[i] > state[i]) {
+				essentialActions[essentialCount++] = i;
+			}
+		}
+		essentialActions.resize(essentialCount);
+		return essentialActions;
+	}
+
 	vector<int> getViableExecutors(int action) {
 		int executorName;
 		if (action == n.chronoboost) {
@@ -680,6 +723,7 @@ public:
 Names ProtossMacroEnv::n;
 double ProtossMacroEnv::REGEN_RATE;
 vector<AssetStats> ProtossMacroEnv::assetList;
+int ProtossMacroEnv::ACTION_SPACE;
 
 namespace py = pybind11;
 
@@ -708,6 +752,9 @@ PYBIND11_MODULE(probotest1, m) {
 		.def("getViableActions", &ProtossMacroEnv::getViableActions)
 		.def("getViableExecutors", &ProtossMacroEnv::getViableExecutors)
 		.def("pickNextExecutor", &ProtossMacroEnv::pickNextExecutor)
+		.def("getTimeAllComplete", &ProtossMacroEnv::getTimeAllComplete)
+		.def("getEssentialActions", &ProtossMacroEnv::getEssentialActions)
+		.def("isDone",&ProtossMacroEnv::isDone)
         .def_readwrite("minerals", &ProtossMacroEnv::minerals)
 		.def_readwrite("gas", &ProtossMacroEnv::gas)
 		.def_readwrite("time", &ProtossMacroEnv::time)
@@ -716,6 +763,7 @@ PYBIND11_MODULE(probotest1, m) {
 		.def_readwrite("probeCountOnMinerals", &ProtossMacroEnv::mProbeCount)
 		.def_readwrite("probeCountOnGas", &ProtossMacroEnv::gProbeCount)
 		.def_readwrite("state", &ProtossMacroEnv::state)
+		.def_readwrite("goalState", &ProtossMacroEnv::goalState)
         ;
 
 	pybind11::class_<Names>(m, "Names")
